package de.fhdw.gaming.ipspiel23.memory;

import java.util.Optional;

/**
 * Interface for the Memory Provider. Returns Memory for a Memory Identifier.
 */
public interface IGameMemoryProvider {

    /**
     * Requests a game memory instance using the specified reuse identifier.
     * If a game memory instance matches the reuse identifier, that existing instance will be returned.
     * Otherwise, a new instance will be created for that reuse identifier.
     * 
     * @param <ROUND_DATA> The type of round data to be stored in the game memory.
     * @param reuseIdentifier The state-bound ID uniquely identifying the strategy instance requesting the game memory.
     * @param capacity The requested capacity of the game memory. The capacity is ignored if a memory is found!
     * @return the new or existing game memory instance.
     */
    <ROUND_DATA> IGameMemory<ROUND_DATA> requestMemoryForStrategy(
        IGameMemoryIdentifier reuseIdentifier, IGameMemoryCapacity capacity);

    /**
     * Tries to request a game memory instance using the specified reuse identifier.
     * If a game memory instance matches the reuse identifier, that existing instance will be returned.
     * Otherwise, an empty optional will be returned.
     * 
     * @param <ROUND_DATA> The type of round data to be stored in the game memory.
     * @param reuseIdentifier The state-bound ID uniquely identifying the strategy instance requesting the game memory.
     * @return the existing game memory instance, or an empty optional if no instance matches the reuse identifier.
     */
    <ROUND_DATA> Optional<IGameMemory<ROUND_DATA>> tryRequestMemoryForStrategy(
        IGameMemoryIdentifier reuseIdentifier);
}